@@ -1,3 +1,4 @@
+import json
 import logging
 import os
 import random
@@ -51,7 +52,7 @@ class SynologyFileCache():
     def create_folder(self, folder_path):
         basename = os.path.basename(folder_path)
         dirname = os.path.dirname(folder_path)
-        self.getFl().create_folder(folder_path=dirname, name=basename)
+        return self.getFl().create_folder(folder_path=dirname, name=basename)
 
     def set(self, key, file_path, del_local=True):
         """
@@ -136,10 +137,13 @@ class SynologyFileCache():
             cache_path = "%s/%s" % (self.cache_path_root, self.namespace)
 
         try:
-            return self.getFl().delete_blocking_function(path=cache_path).get('success', False)
+            return self._delete(path=cache_path)
         except:
             return False
 
+    def _delete(self, path):
+        return self.getFl().delete_blocking_function(path=path).get('success', False)
+
     def exist(self, key):
         """
         判断缓存文件是否存在
@@ -148,3 +152,81 @@ class SynologyFileCache():
         """
         file_path = "%s/%s/%s" % (self.cache_path_root, self.namespace, key)
         return self._exist(file_path=file_path)
+
+    def list_cache_objects(self):
+        """
+        获取文件夹OSS缓存文件
+        """
+        prefix_path = "%s/%s" % (self.cache_path_root, self.namespace)
+        list_dir = []
+        marker = ''
+        begin = False
+        while (marker != '') or (not begin):
+            list_objects = self.bucket.list_objects(prefix=prefix_path, max_keys=300, marker=marker)
+            marker = list_objects.next_marker
+            begin = True
+            for i in list_objects.object_list:
+                list_dir.append(i)
+        return list_dir
+
+    def copy(self, target_namespace, key=None):
+
+        source = os.path.join(self.cache_path_root, self.namespace)
+        target_folder = os.path.join(self.cache_path_root, target_namespace)
+        same_origin = os.path.dirname(source) == os.path.dirname(target_folder)
+
+        if key:
+            source = os.path.join(source, key)
+            same_origin = os.path.dirname(source) == os.path.dirname(target_folder)
+
+        if same_origin:
+            raise Exception('文件来源和目的地不能相同：%s->%s' % (os.path.dirname(source), os.path.dirname(target_folder)))
+
+        if not key:
+            file_path = os.path.join(os.path.dirname(target_folder), os.path.basename(source))
+            if self._exist(file_path=file_path):
+                if not self._delete(path=file_path):
+                    raise Exception('对存在的目标文件执行覆盖失败：%s' % file_path)
+            if self._exist(file_path=target_folder):
+                if not self._delete(path=target_folder):
+                    raise Exception('对存在的目标文件执行覆盖失败：%s' % target_folder)
+
+            before_target_folder = target_folder
+            target_folder = os.path.dirname(target_folder)
+
+            if not self._exist(file_path=target_folder):
+                if not self.create_folder(folder_path=target_folder):
+                    raise Exception('目标文件夹创建失败：%s' % target_folder)
+        else:
+            target_folder_file = os.path.join(target_folder, key)
+            if self._exist(file_path=target_folder_file):
+                if not self._delete(path=target_folder_file):
+                    raise Exception('对存在的目标文件执行覆盖失败：%s' % target_folder_file)
+            if not self._exist(file_path=target_folder):
+                if not self.create_folder(folder_path=target_folder):
+                    raise Exception('目标文件夹创建失败：%s' % target_folder)
+
+        self.getFl().start_copy_move(path=source, dest_folder_path=target_folder)
+
+        def is_finish(taskid):
+            res = self.getFl().get_copy_move_status(taskid=taskid)
+            if res.get('data'):
+                if res.get('data').get('finished'):
+                    return True
+                else:
+                    return is_finish(taskid=taskid)
+
+            else:
+                return False
+
+        if not key:
+            if is_finish(self.getFl()._copy_move_taskid):
+                res = self.getFl().rename_folder(
+                    path=[file_path],
+                    name=[os.path.basename(before_target_folder)]
+                )
+                return res.get('success')
+            else:
+                return False
+        else:
+            return is_finish(self.getFl()._copy_move_taskid)
